


def load_db():
    return get_db_dict()


def get_log_statements_for_scenario(scenario):
    return get_by_key(scenario)


def get_by_key(key):
    the_dict = get_db_dict()
    return the_dict.get(key)


def get_db_dict():
    local_dict = dict()
    local_dict['SO4001'] = get_SO4001_dict()
    local_dict['SO4002'] = get_SO4002_dict()
    local_dict['SO4003'] = get_SO4003_dict()
    local_dict['SO4004'] = get_SO4001_dict()
    local_dict['SO4005'] = get_SO4002_dict()
    local_dict['SO4006'] = get_SO4003_dict()
    return local_dict


def get_SO4001_dict():
    local_dict = dict()
    local_log_list = list()
    local_log_list.append("startHomeRouterJob called")
    local_log_list.append("WorkSchedulingManager Initialized enqueued worker service  is : Home Router Service")
    local_log_list.append("In HomeRouterWorkerService doWork()")
    local_log_list.append("Calling InHomeRouter API...")
    local_log_list.append("POST https://device-qa.cmdev.spectrum.net/device/5.0.0/in-home-router")
    local_log_list.append("200 https://device-qa.cmdev.spectrum.net/device/5.0.0/in-home-router")
    local_log_list.append("InHomeRouter API response=true")
    local_log_list.append("CmSmfo event")
    local_dict['logs'] = local_log_list
    local_dict['counts'] = [1,1,1,1,2,2,1,1]
    return local_dict


def get_SO4002_dict():
    local_dict = dict()
    local_log_list = list()
    local_log_list.append("startHomeRouterJob called")
    local_log_list.append("WorkSchedulingManager Initialized enqueued worker service  is : Home Router Service")
    local_log_list.append("In HomeRouterWorkerService doWork()")
    local_log_list.append("Calling InHomeRouter API...")
    local_log_list.append("POST https://device-qa.cmdev.spectrum.net/device/5.0.0/in-home-router")
    local_log_list.append("403 https://device-qa.cmdev.spectrum.net/device/5.0.0/in-home-router")
    local_log_list.append("InHomeRouter API response=true")
    local_log_list.append("CmSmfo event")
    local_dict['logs'] = local_log_list
    local_dict['counts'] = [1,1,1,1,2,2,0,0]
    return local_dict


def get_SO4003_dict():
    local_dict = dict()
    local_log_list = list()
    local_log_list.append("startHomeRouterJob called")
    local_log_list.append("WorkSchedulingManager Initialized enqueued worker service  is : Home Router Service")
    local_log_list.append("In HomeRouterWorkerService doWork()")
    local_log_list.append("Calling InHomeRouter API...")
    local_log_list.append("POST https://device-qa.cmdev.spectrum.net/device/5.0.0/in-home-router")
    local_log_list.append("403 https://device-qa.cmdev.spectrum.net/device/5.0.0/in-home-router")
    local_log_list.append("InHomeRouter API response=false")
    local_log_list.append("Connection Manager stopped")
    local_log_list.append("Home Router Service  still not scheduled")
    local_log_list.append("Home Router Service Daily  still not scheduled")
    local_dict['logs'] = local_log_list
    local_dict['counts'] = [1, 1, 1, 1, 2, 2, 1, 1, 3, 2]
    return local_dict



import os
import subprocess
import time

ADB_EXECUTION_WAIT = 3
ADB_DEVICES_SCRIPTS = "adb devices"
CMD_WAKE_DEVICE = "adb -s {0} shell input keyevent 224"


def connect_via_adb(serial_num):
    # Connect to the selected device
    print("Waiting for connection ...")
    connect = os.popen("adb -s " + serial_num + " connect").read()
    time.sleep(15)
    if len(connect) == 0:
        return True
    else:
        return False


def run_adb_command(command: str, wait_time=ADB_EXECUTION_WAIT):
    try:
        process = subprocess.run(
            command.split(),
            check=True,
            stdout=subprocess.PIPE,
            universal_newlines=True,
            errors="ignore",
        )
        # Wait for driver to catch up
        time.sleep(wait_time)
        return process.stdout.strip()
    except subprocess.CalledProcessError as error:
        print(f"Failed to run subprocess command: {error}")


def clear_log_cat_logs(serial_num):
    """
    This function clears the logs in the file.
    """
    adb_log_clear = "adb -s {0} logcat -c"
    str = adb_log_clear.format(serial_num)
    run_adb_command(str)


def get_full_log(serial_num, scenario):
    """
    This function collects the logs of CM headless package
    """
    current_dir = os.getcwd()
    file_path = os.path.join(current_dir, "Logs")
    device_log = run_adb_command("adb -s {0} logcat -d".format(serial_num))
    time.sleep(20)
    log_file = os.path.join(file_path, serial_num + "_" + scenario + "_full.txt")
    with open(log_file, "w", encoding="utf-8") as file:
        file.write(device_log)
    return log_file


def get_connectivity_log(serial_num, scenario):
    """
    This function collects the logs of dumpsys connectivity logs
    """
    current_dir = os.getcwd()
    file_path = os.path.join(current_dir, "Logs")
    device_log = run_adb_command("adb -s {0} shell dumpsys connectivity".format(serial_num))
    time.sleep(10)
    log_file = os.path.join(file_path, serial_num + "_" + scenario + "_connectivity.txt")
    with open(log_file, "w", encoding="utf-8") as file:
        file.write(device_log)
    return log_file


def get_os_version(serial_num):
    cmd = "adb -s {0} shell getprop ro.build.version.release"
    os_version = run_adb_command(cmd.format(serial_num))
    print(os_version)
    return os_version



from behave import *
import os
import subprocess
import time

ADB_EXECUTION_WAIT = 3
ADB_DEVICES_SCRIPTS = "adb devices"
CMD_WAKE_DEVICE = "adb -s {0} shell input keyevent 224"


def connect_via_adb(serial_num):
    # Connect to the selected device
    print("Waiting for connection ...")
    connect = os.popen("adb -s " + serial_num + " connect").read()
    time.sleep(15)  #Increased wait time for connection
    if len(connect) == 0:
        return True
    else:
        return False


def run_adb_command(command: str, wait_time=ADB_EXECUTION_WAIT):
    try:
        process = subprocess.run(
            command.split(),
            check=True,
            stdout=subprocess.PIPE,
            universal_newlines=True,
            errors="ignore",
        )
        # Wait for driver to catch up
        time.sleep(wait_time)
        return process.stdout.strip()
    except subprocess.CalledProcessError as error:
        print(f"Failed to run subprocess command: {error}")
        return None #Return None on error


def clear_log_cat_logs(serial_num):
    """
    This function clears the logs in the file.
    """
    adb_log_clear = "adb -s {0} logcat -c"
    str = adb_log_clear.format(serial_num)
    result = run_adb_command(str)
    if result is None:
        raise Exception("Failed to clear logcat logs")


def get_full_log(serial_num, scenario):
    """
    This function collects the logs of CM headless package
    """
    current_dir = os.getcwd()
    file_path = os.path.join(current_dir, "Logs")
    os.makedirs(file_path, exist_ok=True) # Ensure Logs directory exists

    device_log = run_adb_command("adb -s {0} logcat -d".format(serial_num))
    if device_log is None:
        raise Exception("Failed to get full log")

    time.sleep(20)
    log_file = os.path.join(file_path, serial_num + "_" + scenario + "_full.txt")
    with open(log_file, "w", encoding="utf-8") as file:
        file.write(device_log)
    return log_file


def get_connectivity_log(serial_num, scenario):
    """
    This function collects the logs of dumpsys connectivity logs
    """
    current_dir = os.getcwd()
    file_path = os.path.join(current_dir, "Logs")
    os.makedirs(file_path, exist_ok=True) # Ensure Logs directory exists

    device_log = run_adb_command("adb -s {0} shell dumpsys connectivity".format(serial_num))
    if device_log is None:
        raise Exception("Failed to get connectivity log")
    time.sleep(10)
    log_file = os.path.join(file_path, serial_num + "_" + scenario + "_connectivity.txt")
    with open(log_file, "w", encoding="utf-8") as file:
        file.write(device_log)
    return log_file


def get_os_version(serial_num):
    cmd = "adb -s {0} shell getprop ro.build.version.release"
    os_version = run_adb_command(cmd.format(serial_num))
    if os_version is None:
        raise Exception("Failed to get OS version")
    print(os_version)
    return os_version


@given('the Android device with serial number "{serial_number}" is connected')
def step_impl(context, serial_number):
    context.serial_number = serial_number
    success = connect_via_adb(serial_number)
    assert success, f"Failed to connect to device {serial_number}"


@when('the connection fails')
def step_impl(context):
    context.connection_successful = False
    # Simulate a connection failure (replace with actual failure logic if needed)
    # ...


@then('an error message should be displayed')
def step_impl(context):
    assert not context.connection_successful, "No error message displayed when connection failed"



@when('I clear the logcat logs')
def step_impl(context):
    clear_log_cat_logs(context.serial_number)


@when('I wake up the device')
def step_impl(context):
    cmd = CMD_WAKE_DEVICE.format(context.serial_number)
    run_adb_command(cmd)


@when('I get the full log for scenario "{scenario}"')
def step_impl(context, scenario):
    context.full_log_file = get_full_log(context.serial_number, scenario)


@when('I get the connectivity log for scenario "{scenario}"')
def step_impl(context, scenario):
    context.connectivity_log_file = get_connectivity_log(context.serial_number, scenario)


@when('I get the OS version')
def step_impl(context):
    context.os_version = get_os_version(context.serial_number)


@then('the full log file "{log_file}" should exist')
def step_impl(context, log_file):
    assert os.path.exists(log_file), f"Log file {log_file} does not exist"


@then('the connectivity log file "{log_file}" should exist')
def step_impl(context, log_file):
    assert os.path.exists(log_file), f"Log file {log_file} does not exist"


@then('the OS version should be retrieved successfully')
def step_impl(context):
    assert context.os_version is not None, "OS version not retrieved"


import re
import time
from behave import *

from features.common.adb_utils import run_adb_command, connect_via_adb, clear_log_cat_logs, get_full_log, get_connectivity_log


@given('devices are connected')
def get_connected_devices(context):
    formatted_devices_list = []
    connected_devices = run_adb_command('adb devices')
    connected_devices_list = [
        i.split("\t")[0] for i in connected_devices.split("\n")[1:]
    ]
    formatted_devices_list.extend(connected_devices_list)
    successful_list = list()
    for device in formatted_devices_list:
        try:
            result = connect_via_adb(device)
            if not result:
                raise Exception(f"Failed to connect to device: {device}")
            else:
                successful_list.append(device)
        except Exception as e:
            print(f"Error connecting to device: {device} - {e}")
    context.devices = successful_list
    assert len(successful_list) > 0, "No connected devices"


@given('IMEI is present')
def get_imei(context):
    print("begin get_imei")
    device = context.devices[0]
    output = run_adb_command(f"adb -s {0} shell content query --uri content://com.spectrum.cm.headless.library.providers.LibraryServiceProvider/getDynamicImeis".format(device))
    if 'Row: 0 =' in output and (substring := output.split('=')[1].strip()):
        imei_list = [part.split('=')[1].strip() for part in output.split('Row:') if '=' in part]
        integer_list = [int(x) for x in imei_list[0].split(', ')]
        for imei in integer_list:
            context.imei = imei
            break
    print("imei=" + str(context.imei))
    print("end get_imei")
    assert device, "IMEI not available"


@when('logs are clear')
def clear_logs(context):
    device = context.devices[0]
    print("begin clear_logs via adb command")
    clear_log_cat_logs(device)
    print("end clear_logs via adb")


@when('data is cleared')
def clear_data(context):
    device = context.devices[0]
    print("begin clear_data via adb command")
    cmd1 = "adb -s {0} shell pm clear com.spectrum.cm.headless"
    run_adb_command(cmd1.format(device))
    print("end clear_data via adb command")


@when('device is registered')
def device_is_registered(context):
    device = context.devices[0]
    print("begin device_is_registered")
    output = run_adb_command("adb -s {0} shell content query --uri content://com.spectrum.cm.headless.library.providers.LibraryServiceProvider/isRegistered".format(device))
    if 'Row: 0 =' in output and (substring := output.split('=')[1].strip()):
        print(output)
    print("end device_is_registered")


@when('device reboots')
def device_reboots(context):
    device = context.devices[0]
    print("begin device_reboots via adb command")
    cmd = "adb -s {0} reboot"
    run_adb_command(cmd.format(device))
    print("end device_reboots via adb command")


@when('cmc is killed')
def cmc_is_killed(context):
    device = context.devices[0]
    print("begin kill cmc via adb command")
    cmd = "adb -s {0} shell am kill com.spectrum.cm.headless"
    run_adb_command(cmd.format(device))
    print("end kill cmc via adb command")


# @when('install cmc')
# def install_cmc(context):
#     print('Install apk')
#     airlytics_aut = get_aut_path("sok_android")
#     device_name = context.connected_devices[0]
#     cmd1 = "adb -s " + str(device_name) + " install -r -d " + airlytics_aut + "Logs/363.apk"
#     print(cmd1)
#     run_adb_command(cmd1.format(device_name))
#     time.sleep(10)


@then('wait 10')
def wait_10(context):
    time.sleep(10)


@then('wait 120')
def wait_120(context):
    time.sleep(120)


@then('turn wifi off')
def turn_wifi_off(context):
    device = context.devices[0]
    print("begin turn_wifi_off via adb")
    cmd = "adb -s {0} shell svc wifi disable"
    run_adb_command(cmd.format(device))
    wait_10(context)
    status = check_internet_connection(device)
    print("end turn_wifi_off via adb")
    assert status == "0", "Wifi not off and it should be"


@then('turn wifi on')
def turn_wifi_on(context):
    device = context.devices[0]
    print("begin turn_wifi_on via adb")
    cmd = "adb -s {0} shell svc wifi enable"
    run_adb_command(cmd.format(device))
    status = check_internet_connection(context)
    print("end turn_wifi_on via adb")
    assert status == "1", "Wifi not on and it should be"


@then('turn cell off')
def turn_cell_off(context):
    device = context.devices[0]
    print("begin turn_cell_off via adb")
    cmd = "adb -s {0} shell svc data disable"
    run_adb_command(cmd.format(device))
    # TODO what to verify here
    # status = check_internet_connection()
    print("end turn_cell_off via adb")
    # assert status == "0", "Wifi not off"


@then('turn cell on')
def turn_cell_on(context):
    device = context.devices[0]
    print("begin turn_cell_on via adb")
    cmd = "adb -s {0} shell svc data enable"
    run_adb_command(cmd.format(device))
    # TODO what to verify here
    # status = check_internet_connection()
    print("end turn_cell_on via adb")
    # assert status == "1", "Wifi not on"


@when('start cmc')
def start_cmc(context):
    device = context.devices[0]
    print("start cmc application")
    # device_name = context.connected_devices[0]
    cmd = "adb -s {0} shell content query --uri content://com.spectrum.cm.ServiceProvider/start_service"
    output = run_adb_command(cmd.format(device))
    print(output)
    time.sleep(10)


@when('stop cmc')
def stop_cmc(context):
    device = context.devices[0]
    print("stop cmc application")
    cmd = "adb -s {0} shell content query --uri content://com.spectrum.cm.ServiceProvider/stop_service"
    output = run_adb_command(cmd.format(device))
    print(output)
    print("cm client stopped")


# @then('validate cmc started')
# def validate_cmc_is_running(context):
#     test_passed = True
#     device_name = context.connected_devices[0]
#     cmd1 = "adb -s " + str(device_name) + " shell content query --uri content://com.spectrum.cm.ServiceProvider/query_service"
#     output = run_adb_command(cmd1)
#     print(output)
#     if "status=1" not in output:
#         test_passed = False
#     time.sleep(5)
#     assert test_passed, "CM did not start successfully"
#
#
# @then('validate cmc stopped')
# def validate_cmc_is_stopped(context):
#     test_passed = True
#     device_name = context.connected_devices[0]
#     cmd1 = "adb -s " + str(device_name) + " shell content query --uri content://com.spectrum.cm.ServiceProvider/query_service"
#     output = run_adb_command(cmd1)
#     print(output)
#     if "status=0" not in output:
#         test_passed = False
#     time.sleep(5)
#     assert test_passed, "CM did not stop successfully"


@then('run you tube')
def run_you_tube(context):
    device = context.devices[0]
    print("begin run_you_tube")
    cmd = 'adb -s {0} shell am start https://www.youtube.com/watch?v=rUxyKA_-grg&feature=share'
    output = run_adb_command(cmd.format(device))
    print(output)
    print("end run_you_tube")


@then('stop you tube')
def stop_you_tube(context):
    device = context.devices[0]
    print("begin stop_you_tube")
    cmd = 'adb -s {0} shell am force-stop com.google.android.youtube'
    output = run_adb_command(cmd.format(device))
    print(output)
    print("end stop_you_tube")


def check_internet_connection(context):
    device = context.devices[0]
    cmd = "adb -s {0} shell dumpsys wifi | grep \"Wi-Fi is\""
    output = run_adb_command(cmd.format(device))
    print(output)
    if "disabled" in output:
        return "0"
    else:
        return "1"


@then('gather full log')
def gather_full_log(context):
    device = context.devices[0]
    print("begin gather_log")
    full_scenario_name = context.scenario.name
    name = full_scenario_name.split('--')[0].strip()
    dump_path = get_full_log(str(device), str(name))
    context.full_log = dump_path
    print(dump_path)
    print("end gather_log")


@then('gather connectivity log')
def gather_connectivity_log(context):
    device = context.devices[0]
    print("begin gather_connectivity_log")
    dump_path = get_connectivity_log(str(device), str(context.scenario.name))
    context.connectivity_log = dump_path
    print(dump_path)
    print("end gather_connectivity_log")


def find_log_statements(full_log_path, assert_statements):
    not_found_list = []
    for assert_statement in assert_statements:
        with open(full_log_path, "r") as log_data:
            if assert_statement not in log_data.read():
                not_found_list.append(assert_statement)
    return not_found_list


def validate_logs_by_count(full_log_path, log_dict):
    result_list = []
    assert_statements = log_dict['logs']
    assert_counts = log_dict['counts']
    pos = 0
    for assert_statement in assert_statements:
        with open(full_log_path, "r") as log_data:
            the_lines = log_data.readlines()
            count = 0
            for line in the_lines:
                if assert_statement in line:
                    count += 1
        supposed_count = assert_counts[pos]
        pos = pos + 1
        if count != supposed_count:
            result_item = dict()
            result_item['key'] = assert_statement
            result_item['expected'] = supposed_count
            result_item['actual'] = count
            result_list.append(result_item)
    return result_list



from setuptools import setup, find_packages
setup(
name='cmc-adb_utils',
version='0.1.0',
author='spektig',
author_email='ispektor92@gmail.com',
description='Multi ADB connection',
packages=find_packages(),
classifiers=[
'Programming Language :: Python :: 3',
'License :: OSI Approved :: MIT License',
'Operating System :: OS Independent',
],
python_requires='>=3.10',
)
